@@ -3,12 +3,13 @@
 #[cfg(feature = "wasmbus")]
 use crate::wasmbus_model::Wasmbus;
 use crate::{
-    config::LanguageConfig,
+    config::{LanguageConfig, OutputLanguage},
     error::{print_warning, Error, Result},
     gen::{CodeGen, SourceFormatter},
     model::{
-        codegen_rust_trait, get_operation, get_trait, has_default, is_opt_namespace,
-        serialization_trait, value_to_json, wasmcloud_model_namespace, CommentKind, PackageName,
+        codegen_rust_trait, get_operation, get_sorted_fields, get_trait, has_default,
+        is_opt_namespace, serialization_trait, value_to_json, wasmcloud_model_namespace,
+        CommentKind, PackageName, Ty,
     },
     render::Renderer,
     wasmbus_model::{CodegenRust, Serialization},
@@ -50,19 +51,6 @@ struct Declaration(u8, BytesMut);
 
 type ShapeList<'model> = Vec<(&'model ShapeID, &'model AppliedTraits, &'model ShapeKind)>;
 
-// Modifiers on data type
-// This enum may be extended in the future if other variations are required.
-// It's recursively composable, so you could represent &Option<&Value>
-// with `Ty::Ref(Ty::Opt(Ty::Ref(id)))`
-pub(crate) enum Ty<'typ> {
-    /// write a plain shape declaration
-    Shape(&'typ ShapeID),
-    /// write a type wrapped in Option<>
-    Opt(&'typ ShapeID),
-    /// write a reference type: preceeded by &
-    Ref(&'typ ShapeID),
-}
-
 #[derive(Default)]
 pub struct RustCodeGen<'model> {
     /// if set, limits declaration output to this namespace only
@@ -91,6 +79,9 @@ enum MethodArgFlags {
 }
 
 impl<'model> CodeGen for RustCodeGen<'model> {
+    fn output_language(&self) -> OutputLanguage {
+        OutputLanguage::Rust
+    }
     /// Initialize code generator and renderer for language output.j
     /// This hook is called before any code is generated and can be used to initialize code generator
     /// and/or perform additional processing before output files are created.
@@ -107,7 +98,7 @@ impl<'model> CodeGen for RustCodeGen<'model> {
         if let Some(model) = model {
             if let Some(packages) = model.metadata_value("package") {
                 let packages: Vec<PackageName> = serde_json::from_value(value_to_json(packages))
-                    .map_err(|e| Error::Model(format!("invalid metadata format for package, expecting format '[{{namespace:\"org.example\",crate:\"path::module\"}}]':  {}", e.to_string())))?;
+                    .map_err(|e| Error::Model(format!("invalid metadata format for package, expecting format '[{{namespace:\"org.example\",crate:\"path::module\"}}]':  {}", e)))?;
                 for p in packages.iter() {
                     self.packages.insert(p.namespace.to_string(), p.clone());
                 }
@@ -117,13 +108,7 @@ impl<'model> CodeGen for RustCodeGen<'model> {
     }
 
     fn source_formatter(&self) -> Result<Box<dyn SourceFormatter>> {
-        cfg_if::cfg_if! {
-            if #[cfg(not(target_arch = "wasm32"))] {
-                Ok(Box::new(crate::format::RustSourceFormatter::default()))
-            } else {
-                Ok(Box::new(crate::format::NullFormatter::default()))
-            }
-        }
+        Ok(Box::new(crate::format::RustSourceFormatter::default()))
     }
 
     /// Perform any initialization required prior to code generation for a file
@@ -183,7 +168,7 @@ impl<'model> CodeGen for RustCodeGen<'model> {
                 // the base model has minimal dependencies
                 w.write(
                     r#"
-                #![allow(dead_code, clippy::needless_lifetimes)]
+                #![allow(dead_code, unused_imports, clippy::ptr_arg, clippy::needless_lifetimes)]
                 use serde::{{Deserialize, Serialize}};
              "#,
                 );
@@ -210,7 +195,7 @@ impl<'model> CodeGen for RustCodeGen<'model> {
         }
         w.write(&format!(
             "\npub const SMITHY_VERSION : &str = \"{}\";\n\n",
-            model.smithy_version().to_string()
+            model.smithy_version()
         ));
         Ok(())
     }
@@ -253,9 +238,7 @@ impl<'model> CodeGen for RustCodeGen<'model> {
                     )?;
                 }
                 ShapeKind::Structure(strukt) => {
-                    //if !traits.contains_key(&prelude_shape_named(TRAIT_TRAIT).unwrap()) {
                     self.declare_structure_shape(w, id.shape_name(), traits, strukt)?;
-                    //}
                 }
                 ShapeKind::Operation(_)
                 | ShapeKind::Resource(_)
@@ -265,8 +248,10 @@ impl<'model> CodeGen for RustCodeGen<'model> {
             }
             cfg_if::cfg_if! {
                 if #[cfg(feature = "cbor")] {
-                    self.declare_shape_encoder(w, id, shape)?;
-                    self.declare_shape_decoder(w, id, shape)?;
+                    if !traits.contains_key(&prelude_shape_named(TRAIT_TRAIT).unwrap()) {
+                        self.declare_shape_encoder(w, id, shape)?;
+                        self.declare_shape_decoder(w, id, shape)?;
+                    }
                 }
             }
         }
@@ -294,6 +279,7 @@ impl<'model> CodeGen for RustCodeGen<'model> {
         w.write(match kind {
             CommentKind::Documentation => "/// ",
             CommentKind::Inner => "// ",
+            CommentKind::InQuote => "// ", // not applicable for Rust
         });
         w.write(line);
         w.write(b"\n");
@@ -416,7 +402,7 @@ impl<'model> RustCodeGen<'model> {
                         }
                         _ => {
                             if self.namespace.is_none()
-                                || self.namespace.as_ref().unwrap() != wasmcloud_model_namespace()
+                                || self.namespace.as_ref().unwrap() != id.namespace()
                             {
                                 s.push_str(&self.import_core);
                                 s.push_str("::model::");
@@ -431,14 +417,17 @@ impl<'model> RustCodeGen<'model> {
                     s.push_str(&self.to_type_name(&id.shape_name().to_string()));
                 } else {
                     match self.packages.get(&id.namespace().to_string()) {
-                        Some(package) => {
+                        Some(PackageName {
+                            crate_name: Some(crate_name),
+                            ..
+                        }) => {
                             // the crate name should be valid rust syntax. If not, they'll get an error with rustc
-                            s.push_str(&package.crate_name);
+                            s.push_str(crate_name);
                             s.push_str("::");
                             s.push_str(&self.to_type_name(&id.shape_name().to_string()));
                         }
-                        None => {
-                            return Err(Error::Model(format!("undefined create for namespace {} for symbol {}. Make sure codegen.toml includes all dependent namespaces",
+                        _ => {
+                            return Err(Error::Model(format!("undefined crate for namespace {} for symbol {}. Make sure codegen.toml includes all dependent namespaces, and that the dependent .smithy file contains package metadata with crate: value",
                                                             &id.namespace(), &id)));
                         }
                     }
@@ -577,15 +566,9 @@ impl<'model> RustCodeGen<'model> {
         w.write(b"pub struct ");
         self.write_ident(w, id);
         w.write(b" {\n");
-        // sort fields for deterministic output
-        let mut fields = strukt
-            .members()
-            .map(|m| m.to_owned())
-            .collect::<Vec<MemberShape>>();
-        fields.sort_by_key(|f| f.id().to_owned());
+        let (fields, _is_numbered) = get_sorted_fields(id, strukt)?;
         for member in fields.iter() {
             self.apply_documentation_traits(w, member.id(), member.traits());
-
             // use the declared name for serialization, unless an override is declared
             // with `@sesrialization(name: SNAME)`
             let ser_name = if let Some(Serialization {
@@ -596,8 +579,7 @@ impl<'model> RustCodeGen<'model> {
             } else {
                 member.id().to_string()
             };
-            //let declared_name = member.id().to_string();
-            let rust_field_name = self.to_field_name(member.id())?;
+            let rust_field_name = self.to_field_name(member.id(), member.traits())?;
             if ser_name != rust_field_name {
                 w.write(&format!("  #[serde(rename=\"{}\")] ", ser_name));
             }
@@ -689,8 +671,6 @@ impl<'model> RustCodeGen<'model> {
                 }
             }
             let (op, op_traits) = get_operation(model, operation, service_id)?;
-
-            // TODO: re-think what to do if operation is in another namespace and self.namespace is None
             let method_id = operation.shape_name();
             let _flags = self.write_method_signature(w, method_id, op_traits, op)?;
             w.write(b";\n");
@@ -757,7 +737,7 @@ impl<'model> RustCodeGen<'model> {
         method_traits: &AppliedTraits,
         op: &Operation,
     ) -> Result<MethodArgFlags> {
-        let method_name = self.to_method_name(method_id);
+        let method_name = self.to_method_name(method_id, method_traits);
         let mut arg_flags = MethodArgFlags::Normal;
         self.apply_documentation_traits(w, method_id, method_traits);
         w.write(b"async fn ");
@@ -817,7 +797,6 @@ impl<'model> RustCodeGen<'model> {
         );
 
         for method_id in service.operations() {
-            // TODO: if it's valid for a service to include operations from another namespace, then this isn't doing the right thing
             // we don't add operations defined in another namespace
             if let Some(ref ns) = self.namespace {
                 if method_id.namespace() != ns {
@@ -825,35 +804,61 @@ impl<'model> RustCodeGen<'model> {
                 }
             }
             let method_ident = method_id.shape_name();
-            let (op, _) = get_operation(model, method_id, service_id)?;
+            let (op, method_traits) = get_operation(model, method_id, service_id)?;
             w.write(b"\"");
             w.write(&self.op_dispatch_name(method_ident));
             w.write(b"\" => {\n");
-            if op.has_input() {
+            if let Some(op_input) = op.input() {
                 // let value : InputType = deserialize(...)?;
+                #[cfg(feature = "cbor-msg")]
+                w.write(b"let d = &mut minicbor::Decoder::new(&message.arg);\n");
                 w.write(b"let value: ");
-                // TODO: should this be input.target?
-                self.write_type(w, Ty::Shape(op.input().as_ref().unwrap()))?;
-                w.write(b" = deserialize(message.arg.as_ref())\
-                  .map_err(|e| RpcError::Deser(format!(\"message '{}': {}\", message.method, e)))?;\n");
+                self.write_type(w, Ty::Shape(op_input))?;
+                w.write(b" = ");
+                cfg_if::cfg_if! {
+                    if #[cfg(feature = "cbor-msg")] {
+                        w.write(&self.decode_shape_id(op_input)?);
+                        w.write(";\n");
+                    } else {
+                        w.write(b" deserialize(message.arg.as_ref())\
+                            .map_err(|e| RpcError::Deser(format!(\"message '{}': {}\", message.method, e)))?;\n");
+                    }
+                }
             }
             // let resp = Trait::method(self, ctx, &value).await?;
-            w.write(b"let resp = ");
+            if op.output().is_some() {
+                w.write(b"let resp = ");
+            } else {
+                w.write(b"let _resp = ");
+            }
+            let method_name = self.to_method_name(method_ident, method_traits);
             self.write_ident(w, service_id); // Service::method
             w.write(b"::");
-            w.write(&self.to_method_name(method_ident));
+            w.write(&method_name);
             w.write(b"(self, ctx");
             if op.has_input() {
                 w.write(b", &value");
             }
             w.write(b").await?;\n");
 
-            // serialize result
-            w.write(b"let buf = Cow::Owned(serialize(&resp)?);\n");
+            if let Some(_op_output) = op.output() {
+                // serialize result
+                cfg_if::cfg_if! {
+                    if #[cfg(feature = "cbor-msg")] {
+                        w.write(b"let mut buf = Vec::new(); let e = &mut minicbor::Encoder::new(&mut buf);\n");
+                        let s = self.encode_shape_id(_op_output, crate::encode_rust::ValExpr::Plain("resp"))?;
+                        w.write(&s);
+                    } else {
+                        w.write("let buf = serialize(&resp)?;\n");
+                    }
+                }
+            } else {
+                w.write(b"let buf = Vec::new();\n");
+            }
             //w.write(br#"console_log(format!("actor result {}b",buf.len())); "#);
             w.write(b"Ok(Message { method: \"");
             w.write(&self.full_dispatch_name(service_id, method_ident));
-            w.write(b"\", arg: buf })},\n");
+            w.write(b"\", arg: Cow::Owned(buf) })},\n");
         }
         w.write(b"_ => Err(RpcError::MethodNotHandled(format!(\"");
         self.write_ident(w, service_id);
@@ -910,7 +915,6 @@ impl<'model> RustCodeGen<'model> {
         w.write(b"<T> {\n");
 
         for method_id in service.operations() {
-            // TODO: if it's valid for a service to include operations from another namespace, then this isn't doing the right thing
             // we don't add operations defined in another namespace
             if let Some(ref ns) = self.namespace {
                 if method_id.namespace() != ns {
@@ -922,29 +926,57 @@ impl<'model> RustCodeGen<'model> {
             let (op, method_traits) = get_operation(model, method_id, service_id)?;
             w.write(b"#[allow(unused)]\n");
             let arg_flags = self.write_method_signature(w, method_ident, method_traits, op)?;
+            let _arg_is_string = matches!(arg_flags, MethodArgFlags::ToString);
             w.write(b" {\n");
-            if op.has_input() {
-                if matches!(arg_flags, MethodArgFlags::ToString) {
-                    w.write(b"let arg = serialize(&arg.to_string())?;\n");
-                } else {
-                    w.write(b"let arg = serialize(arg)?;\n");
+            if let Some(_op_input) = op.input() {
+                cfg_if::cfg_if! {
+                    if #[cfg(feature = "cbor-msg")] {
+                        if _arg_is_string {
+                            w.write(b"let arg = arg.to_string();\n");
+                        }
+                        w.write(b"let mut buf = Vec::new(); let e = &mut minicbor::Encoder::new(&mut buf);\n");
+                        let s = self.encode_shape_id(_op_input,
+                            if _arg_is_string {
+                                crate::encode_rust::ValExpr::Ref("arg.as_ref()")
+                            } else {
+                                crate::encode_rust::ValExpr::Ref("arg")
+                            }
+                        )?;
+                        w.write(&s);
+                        //let tn = crate::strings::to_snake_case(&self.type_string(Ty::Shape(op.input().as_ref().unwrap()))?);
+                        //w.write(&format!("encode_{}(&mut e, arg)?;", tn));
+                    } else {
+                        if matches!(arg_flags, MethodArgFlags::ToString) {
+                            w.write(b"let buf = serialize(&arg.to_string())?;\n");
+                        } else {
+                            w.write(b"let buf = serialize(arg)?;\n");
+                        }
+                    }
                 }
             } else {
-                w.write(b"let arg = *b\"\";\n");
+                w.write(b"let buf = *b\"\";\n");
             }
             w.write(b"let resp = self.transport.send(ctx, Message{ method: ");
             // note: legacy is just the latter part
             w.write(b"\"");
             w.write(&self.full_dispatch_name(service_id, method_ident));
             //w.write(&self.op_dispatch_name(method_ident));
-            w.write(b"\", arg: Cow::Borrowed(&arg)}, None).await?;\n");
-            if op.has_output() {
-                w.write(
-                    b"let value = deserialize(&resp)\
-                   .map_err(|e| RpcError::Deser(format!(\"response to {}: {}\", \"",
-                );
-                w.write(&method_ident.to_string());
-                w.write(b"\", e)))?; Ok(value)");
+            w.write(b"\", arg: Cow::Borrowed(&buf)}, None).await?;\n");
+            if let Some(_op_output) = op.output() {
+                cfg_if::cfg_if! {
+                    if #[cfg(feature = "cbor-msg")] {
+                        w.write(b"let d = &mut minicbor::Decoder::new(&resp);\n");
+                        w.write(b"let value = ");
+                        w.write(&self.decode_shape_id(_op_output)?);
+                        w.write(b";");
+                    } else {
+                        w.write(b"let value = deserialize(&resp)");
+                        w.write(b".map_err(|e| RpcError::Deser(format!(\"response to {}: {}\", \"");
+                        w.write(&method_ident.to_string());
+                        w.write(b"\", e)))?;");
+                    }
+                }
+                w.write(b"Ok(value)");
             } else {
                 w.write(b"Ok(())");
             }
