@@ -7,7 +7,7 @@ use crate::error::Error;
 use std::path::PathBuf;
 
 /// wraps the logic inside build.rs
-pub fn rust_build<P: Into<PathBuf>> (
+pub fn rust_build<P: Into<PathBuf>>(
     config_path: P,
 ) -> std::result::Result<(), Box<dyn std::error::Error>> {
     use crate::{
@@ -28,7 +28,7 @@ pub fn rust_build<P: Into<PathBuf>> (
     })?;
     let mut config = config_file
         .parse::<CodegenConfig>()
-        .map_err(|e| Error::Build(format!("parsing config: {}", e.to_string())))?;
+        .map_err(|e| Error::Build(format!("parsing config: {}", e)))?;
     config.base_dir = config_path.parent().unwrap().to_path_buf();
     config.output_languages = vec![OutputLanguage::Rust];
 
